"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const deepStrictEqual = require("deep-strict-equal");
const format_1 = require("@cybernaut/utils/lib/format");
const Loggable_1 = require("./Loggable");
class ConditionBuilder extends Loggable_1.Loggable {
    constructor(description, accessor, negated) {
        super(description, ['accessor', 'negated', 'build']);
        this.accessor = accessor;
        this.negated = negated;
    }
    /* tslint:disable-next-line no-any */
    equalTo(value) {
        return this.build('any', this.log, actualValue => {
            if (actualValue !== actualValue && value !== value) {
                return true;
            }
            return deepStrictEqual(actualValue, value);
        });
    }
    above(value) {
        return this.build('number', this.log, actualValue => actualValue > value);
    }
    atLeast(value) {
        return this.build('number', this.log, actualValue => actualValue >= value);
    }
    atMost(value) {
        return this.build('number', this.log, actualValue => actualValue <= value);
    }
    below(value) {
        return this.build('number', this.log, actualValue => actualValue < value);
    }
    between(minValue, maxValue) {
        return this.build('number', this.log, actualValue => actualValue >= minValue && actualValue <= maxValue);
    }
    containing(value) {
        return this.build('string', this.log, actualValue => actualValue.indexOf(value) > -1);
    }
    matching(value) {
        return this.build('string', this.log, actualValue => value.test(actualValue));
    }
    build(type, description, predicate) {
        const { accessor, negated } = this;
        return {
            description,
            accessor,
            predicate: actualValue => {
                if (type !== 'any') {
                    /* tslint:disable-next-line strict-type-predicates */
                    if (typeof actualValue !== type) {
                        throw new Error(`Expected an actual value of type ${type}, got: ${format_1.format(actualValue)}`);
                    }
                }
                return predicate(actualValue);
            },
            negated
        };
    }
}
exports.ConditionBuilder = ConditionBuilder;
//# sourceMappingURL=ConditionBuilder.js.map