"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const format_1 = require("@cybernaut/utils/lib/format");
class Loggable {
    /* istanbul ignore next */
    constructor(description, keysToIgnore = []) {
        const blacklist = new Set([...keysToIgnore, 'log']);
        this.description = this.additionalDescription = description;
        return new Proxy(this, {
            get: (target, property, receiver) => {
                let value;
                if (typeof property !== 'string' || blacklist.has(property)) {
                    value = target[property];
                    return value === target ? receiver : value;
                }
                const additionalDescription = target.additionalDescription;
                target.additionalDescription += `.${property}`;
                value = target[property];
                /* tslint:disable-next-line strict-type-predicates */
                if (typeof value !== 'function') {
                    return value === target ? receiver : value;
                }
                target.additionalDescription = additionalDescription;
                /* tslint:disable-next-line no-any */
                return (...args) => {
                    target.additionalDescription += `.${property}(${args
                        .map(format_1.format)
                        .join(', ')})`;
                    /* tslint:disable-next-line ban-types */
                    return value.apply(receiver, args);
                };
            }
        });
    }
    get log() {
        try {
            return this.additionalDescription;
        }
        finally {
            this.additionalDescription = this.description;
        }
    }
}
exports.Loggable = Loggable;
//# sourceMappingURL=Loggable.js.map